using Model;
using Model.Serialization;
using Model.Stub;
using Newtonsoft.Json.Linq;
using NuGet.Frameworks;
using System;

namespace TestUnitaires
{

    public class TU_Manager
    {
        [Theory]
        [InlineData("Reference")]
        public void TU_Methods(string? test)
        {
            Playlist p = new(test, "PlaceHolder", "place.holder");
            Album album = new(test, "place.holder", new Artist("test"), "PlaceHolder", "PlaceHolder");
            CustomTitle t = new(test, "test. mp3", "Banger", "path");
            Artist artist = new(test);
            Manager m = new(new LinqXmlSerialization(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data")));
            InfoTitle info = new(test, "url.test", "infos", "desc", Genre.POP, 345);
            m.AddCustomTitle(t);
            m.AddPlaylist(p);
            m.AddAlbum(album);
            m.AddArtist(artist);
            m.AddInfoTitle(info);
            Assert.Contains(t, m.CustomTitles);
            Assert.Contains(album, m.Albums);
            Assert.Contains(p, m.Playlists);
            Assert.Contains(info, m.InfoTitles);
            Assert.Contains(artist, m.Artists);
            m.RemovePlaylist(p);
            m.RemoveAlbum(album);
            m.RemoveCustomTitle(t);
            m.RemoveInfoTitle(info);
            Assert.DoesNotContain(t, m.CustomTitles);
            Assert.DoesNotContain(p, m.Playlists);
            Assert.DoesNotContain(album, m.Albums);
            Assert.DoesNotContain(info, m.InfoTitles);
            m.AddCustomTitle(t);
            m.AddPlaylist(p);
            m.AddAlbum(album);
            m.AddArtist(artist);
            m.AddInfoTitle(info);
            Assert.Equal(m.GetAlbums(), m.Albums);
            Assert.Equal(m.GetInfoTitles(), m.InfoTitles);
            Assert.Equal(m.GetCustomTitles(), m.CustomTitles);
            Assert.Equal(m.GetPlaylists(), m.Playlists);
            Assert.Equal(m.GetArtists(), m.Artists);
            Assert.Equal(m.GetAlbumByName(test), album);
            Assert.Equal(m.GetPlaylistByName(test), p);
            Assert.Equal(m.GetInfoTitleByName(test), info);
            Assert.Equal(m.GetArtistByName(test), artist);

            var ca = m.CurrentAlbum;
            var ci = m.CurrentInfoTitle;
            var cc = m.CurrentPlaying;
            var cp = m.CurrentPlaylist;

            Assert.Equal(m.CurrentAlbum, ca);
            Assert.Equal(m.CurrentInfoTitle, ci);
            Assert.Equal(m.CurrentPlaying, cc);
            Assert.Equal(m.CurrentPlaylist, cp);

            m.CurrentPlaylist = p;
            m.CurrentInfoTitle = info;
            m.CurrentPlaying = t;
            m.CurrentAlbum = album;

            Assert.Equal(p, m.CurrentPlaylist);
            Assert.Equal(info, m.CurrentInfoTitle);
            Assert.Equal(t, m.CurrentPlaying);
            Assert.Equal(album, m.CurrentAlbum);

            m.NextTitle();
            Assert.Null(m.CurrentPlaying);
            m.PreviousTitle();
            Assert.Null(m.CurrentPlaying);

            m.CurrentPlaylist = null;
            m.NextTitle();
            Assert.Null(m.CurrentPlaying);
            m.PreviousTitle();
            Assert.Null(m.CurrentPlaying);

            var cct = m.CurrentTitle();
            Assert.Null(cct);

            m.Loop();

            m.Shuffle();

            p.AddTitle(t);
            m.CurrentPlaylist = p;
            m.CurrentPlaying = t;
            cct = m.CurrentTitle();
            Assert.NotNull(cct);

            var loop = m.CurrentPlaylist.LoopTitle;
            m.Loop();
            Assert.NotEqual(loop, m.CurrentPlaylist.LoopTitle);

            var shuffle = m.CurrentPlaylist.Shuffle;
            m.Shuffle();
            Assert.NotEqual(shuffle, m.CurrentPlaylist.Shuffle);

            var custom = m.GetCustomTitleByPath(t.Path);
            Assert.Equal(t, custom);

            p.AddTitle(t);
            m.RemoveCustomTitleFromPlaylists(t);
            Assert.DoesNotContain(t, p.Titles);
        }
    }
    
}